<?php
use JetBrains\PhpStorm\NoReturn;

require 'app-config.php';

session_start([
    'name'            => 'FRCSESSION',
    'use_strict_mode' => true,
    'cookie_httponly' => true,
    'cookie_samesite' => 'Strict']);

/**
 * Add a message to be displayed at the top of the page
 *
 * @param string $level The level (type) of the message
 * @param string $message The message itself
 */
function add_message(string $level, string $message): void {
    if (!array_key_exists(Key::USER_MSG, $_REQUEST)) $_REQUEST[Key::USER_MSG] = array();
    $_REQUEST[Key::USER_MSG][] = ['level' => $level, 'message' => $message];
}

/**
 * Add an error message to be displayed at the top of the page
 *
 * @param string $message The message to be displayed
 */
function add_error(string $message): void {
    add_message('ERROR', $message);
}

/**
 * Add an error message to be displayed at the top of the page
 *
 * @param string $message The message to be displayed
 */
function add_info(string $message): void {
    add_message('INFO', $message);
}

/**
 * Render the page title
 * @param string $title The title of the page being displayed
 */
function page_head(string $title): void {
    $version = match (true) {
        str_ends_with(FRC_VERSION, '.0.0') => substr(FRC_VERSION, 0, strlen(FRC_VERSION) - 4),
        str_ends_with(FRC_VERSION, '.0')   => substr(FRC_VERSION, 0, strlen(FRC_VERSION) - 2),
        default                            => FRC_VERSION
    } ?>
<!DOCTYPE html>
<html lang=en>
<head>
    <meta name=viewport content="width=device-width, initial-scale=1">
    <title><?=$title?> | Feed Reader Central</title>
    <link href=/assets/style.css rel=stylesheet>
</head>
<body>
<header>
    <div><a class=title href="/">Feed Reader Central</a><span class=version>v<?=$version?></span></div>
    <div><?php
        if (array_key_exists(Key::USER_ID, $_SESSION)) {
            echo '<a href=/feed?id=new>Add Feed</a> | <a href=/docs/>Docs</a> | <a href=/user/log-off>Log Off</a>';
            if ($_SESSION[Key::USER_EMAIL] != Security::SINGLE_USER_EMAIL) echo " | {$_SESSION[Key::USER_EMAIL]}";
        } else {
            echo '<a href=/user/log-on>Log On</a> | <a href=/docs/>Docs</a>';
        } ?>
    </div>
</header>
<main hx-target=this><?php
    foreach ($_REQUEST[Key::USER_MSG] ?? [] as $msg) { ?>
        <div>
            <?=$msg['level'] == 'INFO' ? '' : "<strong>{$msg['level']}</strong><br>"?>
            <?=$msg['message']?>
        </div><?php
    }
}

/**
 * Render the end of the page
 */
function page_foot(): void {
    echo '</main></body></html>';
    session_commit();
}

/**
 * Redirect the user to the given URL
 *
 * @param string $value A local URL to which the user should be redirected
 */
#[NoReturn]
function frc_redirect(string $value): void {
    if (str_starts_with($value, 'http')) {
        http_response_code(400);
        die();
    }
    session_commit();
    header("Location: $value", true, 303);
    die();
}

/**
 * Convert a date/time string to a date/time in the configured format
 *
 * @param string $value The date/time string
 * @return string The standard format of a date/time, or '(invalid date)' if the date could not be parsed
 */
function date_time(string $value): string {
    try {
        return (new DateTimeImmutable($value))->format(DATE_TIME_FORMAT);
    } catch (Exception) {
        return '(invalid date)';
    }
}
